#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "mysqlenum.h"

#define STRCMP(str1,str2)   strncmp(str1,str2,strlen(str2))

/*
 * Allocs data and returns the memory pointer
 */
char *alloc_value (char *value)
{
    char *mvalue;
    int value_size;
    
    value_size = strlen(value);
    mvalue = malloc(value_size + 1);
    bzero(mvalue, value_size + 1);
    memcpy(mvalue, value, value_size);
    return mvalue;
}

/*
 * Populates the config structure
 * It's a pointers game... :)
 */
void param_set(struct configuration *conf, char *param, char *value)
{
    int TYPE_STRING = 1,
        TYPE_INT = 2,
        i,
        found = 0;
    
    /* Configuration file parameters */
    void *parameters[] = {
        /* name, pointer to var pointer, type */
        "host", &conf->host, &TYPE_STRING,
        "port", &conf->port, &TYPE_INT,
        "page", &conf->page, &TYPE_STRING,
        "vulnparam", &conf->vulnparam, &TYPE_STRING,
        "otherparams", &conf->otherparams, &TYPE_STRING,
        "matchstring", &conf->matchstring, &TYPE_STRING,
        "sqlquery", &conf->sqlquery, &TYPE_STRING,
        "queryappend", &conf->queryappend, &TYPE_STRING,
        "httpmethod", &conf->httpmethod, &TYPE_INT,
        "httpauth", &conf->httpauth, &TYPE_STRING,
        "httpcookie", &conf->httpcookie, &TYPE_STRING,
        "encodestrings", &conf->encodestrings, &TYPE_INT,
        "delay", &conf->delay, &TYPE_INT,
        "charstart", &conf->charstart, &TYPE_INT,
        "charend", &conf->charend, &TYPE_INT,
        "useragent", &conf->useragent, &TYPE_STRING,
        "customheader", &conf->customheader, &TYPE_STRING,
        "multirowsenum", &conf->multirowsenum, &TYPE_INT,
        "inversematch", &conf->inversematch, &TYPE_INT,
        "startrow", &conf->startrow, &TYPE_INT,
        "encoding", &conf->encoding, &TYPE_INT,
        0,0,0
    };

    for (i=0; ;i+=3)
    {
        char *pname;
        void **ppointer;
        int ptype;
                
        if (parameters[0+i] == 0) break;
        
        pname = (char *)parameters[0+i];
        ppointer = parameters[1+i];
        ptype = *(int *)parameters[2+i];

        /* String parameter... */
        if ((strncmp (pname, param, strlen(pname)) == 0) && (ptype == TYPE_STRING))
        {
            *ppointer = alloc_value(value);
            found = 1;
        }
        
        /* Int parameter... */
        else 
        if ((strncmp (pname, param, strlen(pname)) == 0) && (ptype == TYPE_INT))
        {
        
            if (strncasecmp(value, "POST",4) == 0)
                *(int *)ppointer = HTTP_POST;
            else 
            if (strncasecmp(value, "GET",3) == 0)
                *(int *)ppointer = HTTP_GET;
            else 
            if (strncasecmp(value, "YES",3) == 0)
                *(int *)ppointer = YES;
            else 
            if (strncasecmp(value, "NO",2) == 0)
                *(int *)ppointer = NO;
            else
                *(int *)ppointer = atoi(value);

            found = 1;                
        }
    }
    
    if (!found)
    {
        fprintf(stderr, "\n *** Unrecognized parameter in configuration file: %s\n", param);
        exit (EXIT_FAILURE);
    }

}

/*
 * Setting up default configuration values...
 */
void init_config_values(struct configuration *conf)
{
    conf->encodestrings = NO;
    conf->port = 80;
    conf->httpmethod = HTTP_GET;
    conf->delay = 0;
    conf->charstart = 32;
    conf->charend = 122;
    conf->startrow = 0;
    conf->encoding = 1;
    
    conf->host = "";
    conf->page = "";
    conf->sqlquery = "";
    conf->matchstring = "";
    conf->httpauth = "";
    conf->vulnparam = "";
    conf->otherparams = "";
    conf->queryappend = "--";
    conf->httpcookie = "";
    conf->useragent = "";
    conf->customheader = "";
    conf->multirowsenum = YES;
    conf->inversematch = NO;
}

void check_required_parameter (const char *name, char *value)
{
    if ((value == NULL) || (str_empty(value)))
    {
        fprintf(stderr, "\n *** Required parameter in configuration file not found: %s\n", name);
        exit(EXIT_FAILURE);
    }
}

/*
 * Configuration file parsing
 * OK: 1
 * KO: 0
 */
int parse_config_file(const char *config_file, struct configuration *conf)
{
    FILE *fp;
    char line[1024];
    
    fp = fopen (config_file, "r");
    if (fp == NULL) 
    {
        perror("\n *** Error reading configuration file");
        return 0;
    }

    init_config_values(conf);

    //Reading file lines
    while (fgets(line, sizeof(line)-1, fp) != NULL)
    {
        char *trimline,
             *separator,
             *param,
             *value;
               
        trimline = trim(line);

        if ((!trimline) || (str_empty(trimline)) || (*trimline == '#'))
            continue;
                    
        separator = strchr(trimline, '=');
        if (!separator) continue;
 
        //Getting param name
        *separator = 0;
        param = trim(trimline);
        
        //Getting param value
        value = trim(separator + 1);

        if ((!value) || (str_empty(value)))
            continue;

        param_set(conf, param, value);
    }

    /* checking required parameters */    
    check_required_parameter("host", conf->host);
    check_required_parameter("page", conf->page);
    check_required_parameter("matchstring", conf->matchstring);
    check_required_parameter("vulnparam", conf->vulnparam);
    
    fclose(fp);
    return 1;
}
