/*
 * mySQLenum
 *  
 * Copyright (C) 2009 Andrea Fabrizi <andrea.fabrizi@gmail.com>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "mysqlenum.h"

/*
 * Verifica se una stringa e' costituita esclusivamente
 * da caratteri numerici
 * Ritorna:
 *  OK: 1
 *  KO: -1
 */
int is_num(const char *str)
{
    int i;
    
    for (i=0; i<strlen(str); i++)
        if (!isdigit(str[i])) return -1;
        
    return 1;
}

/*
 * Sostituisce la prima occorrenza di 'search' con 'replace' all'interno di 'buffer'
 * Alloca automaticamente la memoria necessaria e ne restituisce il puntatore
 * Ritorna:
 * OK: puntatore alla memoria allocata
 * KO: NULL 
 *
 */
char *str_replace(const char *search, const char *replace, const char *buffer)
{
    char *needle,
         *result;
    int needle_len,
        buffer_len,
        result_len,
        replace_len,
        pre_len,  //porzione prima di needle
        post_len; //porzione dopo needle

    /* individuo la stringa nel buffer */
    needle = strstr(buffer, search);
    if (needle != NULL)
    {
        needle_len = strlen(search);
        buffer_len = strlen(buffer);
        replace_len = strlen(replace);
        pre_len = needle - buffer;
        post_len = buffer_len - pre_len - needle_len;
        result_len = buffer_len - needle_len + replace_len + 1;
        
        result = malloc(result_len);
        bzero(result,result_len);
    
        /* copio in result la porzione di buffer subito prima di needle */
        strncat(result, buffer, pre_len);

        /* copio in result la stringa sostituita */
        strncat(result, replace, replace_len);

        /* copio in result la porzione di buffer subito dopo needle */
        strncat(result, buffer + pre_len + needle_len, post_len);
        
        return result;    
    }    
    else 
        return NULL;
}


/*
 * Spinner 
 */
void spinner(void)
{
    if (nospinner) return;
    
    static int s = 0;
    char c = ' ';
    
    switch (s)
    {
        case 0: c = '|'; break;
        case 1: c = '/'; break;
        case 2: c = '-'; break;
        case 3: c = '\\'; s=-1; break;
    }
    
    fprintf(stdout,"\033[1D%c", c);
    fflush(stdout);
    s++;
}

/*
 * Stampa un messaggio in base al livello di verbose
 */
void debug_msg(unsigned short int verbose_level, const char* fmt, ...)
{
    va_list ap;

    if (verbose_level <= verbose)
    {
        fprintf(stdout, "\n DEBUG: ");
        va_start(ap, fmt);
        vfprintf(stdout, fmt, ap); 
        va_end(ap);
    }
}

int str_empty (char *str)
{
    if (*str)
        return 0;
    else return 1;
}

int is_space(char c)
{
    if ((c == 0x20) || 
        (c == 0x09) || 
        (c == 0x0A) ||
        (c == 0x0B) || 
        (c == 0x0C) ||  
        (c == 0x0D))
    return 1;
    else return 0;
}

/*
 * Trim a string
 */
char *trim (char *string)
{
    int i;
    
    if (!string)
        return string;

    if (str_empty(string))
        return string;

    //ltrim
    while (is_space(*string))
        string++;

    //rtrim
    for (i=strlen(string) - 1; i>=0; i--)
    {
        if (is_space(string[i]))
           string[i] = 0;
        else
           break; 
    }
    
    return string;
}

